# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django's standard views module
## for FM module
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from __future__ import with_statement
from django.shortcuts import get_object_or_404
from django.contrib.auth.decorators import permission_required
from noc.lib.render import render,render_plain_text,render_success,render_failure,render_json
from noc.lib.fileutils import temporary_file
from noc.fm.models import Event,EventData,EventClassificationRule,EventClassificationRE,EventPriority, EventClass,\
    MIB, MIBRequiredException, EventCategory, EVENT_STATUS_CHOICES, EventPostProcessingRule, EventPostProcessingRE
from noc.sa.models import ManagedObject
from django.http import HttpResponseRedirect,HttpResponseForbidden, HttpResponse
from django.core.paginator import Paginator, InvalidPage, EmptyPage
from django import forms
import random,re
from noc.lib.widgets import AutoCompleteTextInput,lookup
from noc.lib.sysutils import refresh_config
from django.forms.widgets import HiddenInput

##
## Returns Managed object names completion
##
@permission_required("fm.change_event")
def lookup_managed_object(request):
    def lookup_function(q):
        for m in ManagedObject.objects.filter(name__startswith=q):
            yield m.name
    return lookup(request,lookup_function)
##
## Event searching AJAX handler.
## Returns:
## { 
##    count:
##    page:
##    pages:
##    events: [list of events]
## }
##
PAGE_SIZE=20
@permission_required("fm.change_event")
def lookup_events(request):
    events=Event.objects
    page=0
    if request.GET:
        form=EventSearchForm(request.GET)
        if form.is_valid():
            if form.cleaned_data["page"]:
                page=form.cleaned_data["page"]-1
            if form.cleaned_data["from_time"]:
                events=events.filter(timestamp__gte=form.cleaned_data["from_time"])
            if form.cleaned_data["to_time"]:
                events=events.filter(timestamp__lte=form.cleaned_data["to_time"])
            if form.cleaned_data["managed_object"]:
                try:
                    mo=ManagedObject.objects.get(name=form.cleaned_data["managed_object"])
                    events=events.filter(managed_object=mo)
                except ManagedObject.DoesNotExist:
                    pass
            if form.cleaned_data["event_class"]:
                events=events.filter(event_class=form.cleaned_data["event_class"])
            if form.cleaned_data["status"]:
                events=events.filter(status=form.cleaned_data["status"])
            if form.cleaned_data["event_priority"]:
                events=events.filter(event_priority__priority__gte=form.cleaned_data["event_priority"].priority)
            if form.cleaned_data["event_category"]:
                events=events.filter(event_category=form.cleaned_data["event_category"])
            if form.cleaned_data["subject"]:
                events=events.filter(subject__icontains=form.cleaned_data["subject"])
    count=events.count()
    return render_json({
        "count" : count,
        "page"  : page,
        "pages" : count/PAGE_SIZE,
        "events": [[e.event_priority.css_style_name,e.id,e.managed_object.name,str(e.timestamp),e.status,\
                    e.event_category.name,e.event_class.name,e.event_priority.name,e.subject]\
                    for e in events.order_by("-timestamp")[PAGE_SIZE*page:PAGE_SIZE*(page+1)]]
    }
    )

##
## Event search form
##
class EventSearchForm(forms.Form):
    page=forms.IntegerField(required=False,min_value=0,widget=HiddenInput)
    from_time=forms.DateTimeField(required=False,input_formats=["%d.%m.%Y %H:%M:%S"])
    to_time=forms.DateTimeField(required=False,input_formats=["%d.%m.%Y %H:%M:%S"])
    managed_object=forms.CharField(required=False,widget=AutoCompleteTextInput("/fm/lookup/managed_object/"))
    event_category=forms.ModelChoiceField(required=False,queryset=EventCategory.objects.all())
    event_class=forms.ModelChoiceField(required=False,queryset=EventClass.objects.all())
    status=forms.ChoiceField(required=False,choices=[("","---------")]+EVENT_STATUS_CHOICES)
    event_priority=forms.ModelChoiceField(required=False,queryset=EventPriority.objects.all())
    subject=forms.CharField(required=False)
##
## Display events list scheet
##
@permission_required("fm.change_event")
def index(request):
    form=EventSearchForm(initial={"status":"A"})
    return render(request,"fm/index.html",{"form":form})
##
## Dynamically generated CSS for event list priorities
##
def event_list_css(request):
    text="\n\n".join([p.css_style for p in EventPriority.objects.all()])
    return HttpResponse(text,mimetype="text/css")
##
## Event preview
##
@permission_required("fm.change_event")
def event(request,event_id):
    event=get_object_or_404(Event,id=int(event_id))
    return render(request,"fm/event.html",{"e":event})
##
## Drop classification mask on event so it can be classified again
##
@permission_required("fm.change_event")
def reclassify_event(request,event_id):
    event=get_object_or_404(Event,id=int(event_id))
    event.reclassify_event("Reclassification requested by '%s'"%request.user.username)
    link=request.META.get("HTTP_REFERER","/fm/%d/"%event.id)
    return HttpResponseRedirect(link)
##
## Change event status to Open
##
@permission_required("fm.change_event")
def open_event(request,event_id):
    event=get_object_or_404(Event,id=int(event_id))
    event.open_event("Event opened by '%s'"%request.user.username)
    link=request.META.get("HTTP_REFERER","/fm/%d/"%event.id)
    return HttpResponseRedirect(link)
##
## Change event status to Close
##
@permission_required("fm.change_event")
def close_event(request,event_id):
    event=get_object_or_404(Event,id=int(event_id))
    event.close_event("Event closed by '%s'"%request.user.username)
    link=request.META.get("HTTP_REFERER","/fm/%d/"%event.id)
    return HttpResponseRedirect(link)
##
## Create event classification rule from event
##
@permission_required("fm.add_eventclassificationrule")
def create_rule(request,event_id):
    def re_q(s):
        return s.replace("\\","\\\\").replace(".","\\.").replace("+","\\+").replace("*","\\*")
    event=get_object_or_404(Event,id=int(event_id))
    rule=EventClassificationRule(event_class=event.event_class,name="Rule #%d:%d"%(event.id,random.randint(0,100000)),preference=1000)
    rule.save()
    for d in event.eventdata_set.filter(type__in=[">","R"]):
        r=EventClassificationRE(rule=rule,left_re="^%s$"%re_q(d.key)[:254],right_re="^%s$"%re_q(d.value)[:254])
        r.save()
    return HttpResponseRedirect("/admin/fm/eventclassificationrule/%d/"%rule.id)
##
## Clone classification rule from existing one
##
@permission_required("fm.add_eventclassificationrule")
def clone_rule(request,rule_id):
    rule=get_object_or_404(EventClassificationRule,id=int(rule_id))
    # Find suitable rule name
    i=1
    while True:
        name=rule.name+" copy #%d"%i
        try:
            EventClassificationRule.objects.get(name=name)
        except EventClassificationRule.DoesNotExist:
            break
        i+=1
    # Create cloned rule
    new_rule=EventClassificationRule(
        name=name,
        event_class=rule.event_class,
        preference=rule.preference,
        drop_event=rule.drop_event,
        is_builtin=False)
    new_rule.save()
    for r in rule.eventclassificationre_set.all():
        new_r=EventClassificationRE(rule=new_rule,left_re=r.left_re,right_re=r.right_re)
        new_r.save()
    return HttpResponseRedirect("/admin/fm/eventclassificationrule/%d/"%new_rule.id)
##
## Create post-processing rule from event
##
@permission_required("fm.add_eventprostprocessingrule")
def create_postprocessing_rule(request,event_id):
    def re_q(s):
        return s.replace("\\","\\\\").replace(".","\\.").replace("+","\\+").replace("*","\\*")
    event=get_object_or_404(Event,id=int(event_id))
    rule=EventPostProcessingRule(event_class=event.event_class,name="Rule #%d:%d"%(event.id,random.randint(0,100000)),preference=1000)
    rule.save()
    for d in event.eventdata_set.filter(type="V"):
        r=EventPostProcessingRE(rule=rule,var_re="^%s$"%re_q(d.key)[:254],value_re="^%s$"%re_q(d.value)[:254])
        r.save()
    return HttpResponseRedirect("/admin/fm/eventpostprocessingrule/%d/"%rule.id)
##
## Event classification rules overview sheet
##
@permission_required("fm.change_eventclassificationrule")
def view_rules(request):
    return render(request,"fm/view_rules.html",{"rules":EventClassificationRule.objects.order_by("preference")})
##
## Convert event class description to python code
##
@permission_required("fm.add_eventclass")
def py_event_class(request,event_class_id):
    event_class=get_object_or_404(EventClass,id=int(event_class_id))
    return render_plain_text(event_class.python_code)
##
## Convert event classification rule to python code
##
@permission_required("fm.add_eventclassificationrule")
def py_event_classification_rule(request,rule_id):
    rule=get_object_or_404(EventClassificationRule,id=int(rule_id))
    return render_plain_text(rule.python_code)
##
## MIB Upload
##
class MIBUploadForm(forms.Form):
    file=forms.FileField()

@permission_required("fm.add_mib")
def upload_mib(request):
    if request.method=="POST":
        form = MIBUploadForm(request.POST, request.FILES)
        if form.is_valid():
            with temporary_file(request.FILES['file'].read()) as path:
                try:
                    mib=MIB.load(path)
                except MIBRequiredException,x:
                    return render_failure(request,"Failed to upload MIB","%s requires %s"%(x.mib,x.requires_mib))
            return HttpResponseRedirect("/admin/fm/mib/%d/"%mib.id)
    else:
        form=MIBUploadForm()
    return render(request,"fm/mib_upload.html",{"form":form})
##
## Reload noc-classifier config
##
@permission_required("fm.add_eventclassificationrule")
def reload_classifier_config(request):
    referer=request.META.get("HTTP_REFERER","/admin/fm/eventclassificationrule/")
    refresh_config("noc-classifier")
    return HttpResponseRedirect(referer)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Binary data to string encoder/decoder
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import re
##
## Patterns
##
rx_unqoute=re.compile(r"\\x([0-9a-f][0-9a-f])",re.MULTILINE|re.DOTALL)
## Map to convert two-char hex to integer
hex_map=dict([("%02x"%i,chr(i)) for i in range(256)])
##
## Quote binary data to ASCII-string
##
def bin_quote(s):
    def qc(c):
        if c=="\\":
            return "\\\\"
        oc=ord(c)
        if oc<32 or oc>126:
            return "\\x%02x"%oc
        return c
    return "".join([qc(c) for c in s])
    
##
## Decode ASCII-encoded string back to binary
##
def bin_unquote(s):
    """
    >>> [i for i in range(256) if bin_unquote(bin_quote(chr(i)))!=chr(i)]
    []
    """
    return rx_unqoute.sub(lambda x:hex_map[x.group(1)], str(s).replace(r"\\","\\x5c"))
##
## Unit Test
##
if __name__ == "__main__":
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## QoS classification rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.Cisco.IOS.qos import *
##
## Cisco.IOS AutoQoS Trust Device Detected SYSLOG
##
class Cisco_IOS_AutoQoS_Trust_Device_Detected_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS AutoQoS Trust Device Detected SYSLOG"
    event_class=CiscoIOSAutoQoSTrustDeviceDetected
    preference=1000
    patterns=[
        (r"^message$",r"%SWITCH_QOS_TB-5-TRUST_DEVICE_DETECTED: (?P<device>.+?) detected on port (?P<interface>.+?),"),
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
    ]
##
## Cisco.IOS AutoQoS Trust Device Lost SYSLOG
##
class Cisco_IOS_AutoQoS_Trust_Device_Lost_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS AutoQoS Trust Device Lost SYSLOG"
    event_class=CiscoIOSAutoQoSTrustDeviceLost
    preference=1000
    patterns=[
        (r"^message$",r"%SWITCH_QOS_TB-5-TRUST_DEVICE_LOST: (?P<device>.+?) no longer detected on port (?P<interface>.+?),"),
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django URL dispatcher for module IP
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.conf.urls.defaults import *
from django.contrib.auth.decorators import login_required
from noc.ip.views import index,vrf_index,allocate_block,deallocate_block,assign_address,revoke_address,block_tools,download_ips,\
    upload_ips,upload_axfr

urlpatterns = patterns ( "",
    (r"^$",                   login_required(index)),
    (r"^(?P<vrf_id>\d+)/$",   login_required(vrf_index)),
    (r"^(?P<vrf_id>\d+)/(?P<prefix>\d{1,3}\.\d{1,3}\.\d{1,3}.\d{1,3}/\d{1,2})/$",   login_required(vrf_index)),
    (r"^(?P<vrf_id>\d+)/allocate_block/$", login_required(allocate_block)),
    (r"^(?P<vrf_id>\d+)/(?P<prefix>\d{1,3}\.\d{1,3}\.\d{1,3}.\d{1,3}/\d{1,2})/allocate_block/$",   login_required(allocate_block)),
    (r"^(?P<vrf_id>\d+)/(?P<prefix>\d{1,3}\.\d{1,3}\.\d{1,3}.\d{1,3}/\d{1,2})/deallocate_block/$", login_required(deallocate_block)),
    (r"^(?P<vrf_id>\d+)/assign_address/$", login_required(assign_address)),
    (r"^(?P<vrf_id>\d+)/(?P<ip>\d{1,3}\.\d{1,3}\.\d{1,3}.\d{1,3})/assign_address/$", login_required(assign_address)),
    (r"^(?P<vrf_id>\d+)/(?P<ip>\d{1,3}\.\d{1,3}\.\d{1,3}.\d{1,3})/revoke_address/$", login_required(revoke_address)),
    (r"^(?P<vrf_id>\d+)/(?P<prefix>\d{1,3}\.\d{1,3}\.\d{1,3}.\d{1,3}/\d{1,2})/tools/$", login_required(block_tools)),
    (r"^(?P<vrf_id>\d+)/(?P<prefix>\d{1,3}\.\d{1,3}\.\d{1,3}.\d{1,3}/\d{1,2})/download_ips/$", login_required(download_ips)),
    (r"^(?P<vrf_id>\d+)/(?P<prefix>\d{1,3}\.\d{1,3}\.\d{1,3}.\d{1,3}/\d{1,2})/upload_ips/$", login_required(upload_ips)),
    (r"^(?P<vrf_id>\d+)/(?P<prefix>\d{1,3}\.\d{1,3}\.\d{1,3}.\d{1,3}/\d{1,2})/upload_axfr/$", login_required(upload_axfr)),
)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Various text-processing utilities
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import re

##
## Parse string containing table an return a list of table rows.
## Each row is a list of cells.
## Columns are determined by a sequences of ---- or ==== which are
## determines rows bounds.
## Examples:
## First Second Third
## ----- ------ -----
## a     b       c
## ddd   eee     fff
## Will be parsed down to the [["a","b","c"],["ddd","eee","fff"]]
##
rx_header_start=re.compile(r"^\s*[-=]+\s+[-=]+")
rx_col=re.compile(r"^(\s*)([\-]+|[=]+)")
def parse_table(s):
    columns=None
    r=[]
    columns=[]
    for l in s.splitlines():
        if not l.strip():
            columns=[]
            continue
        if rx_header_start.match(l): # Column delimiters found. try to determine column's width
            columns=[]
            x=0
            while l:
                match=rx_col.match(l)
                if not match:
                    break
                columns.append((x+len(match.group(1)),x+len(match.group(1))+len(match.group(2))))
                x+=match.end()
                l=l[match.end():]
        elif columns: # Fetch cells
            r.append([l[f:t].strip() for f,t in columns])
    return r
##
## Convert HTML to plain text
##
rx_html_tags=re.compile("</?[^>+]+>",re.MULTILINE|re.DOTALL)
def strip_html_tags(s):
    t=rx_html_tags.sub("",s)
    for k,v in [("&nbsp;"," "),("&lt;","<"),("&gt;",">"),("&amp;","&")]:
        t=t.replace(k,v)
    return t



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Administrative interface for KB application
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.contrib import admin
from django import forms
from noc.kb.models import *

##
## Admin for Categories
##
class KBCategoryAdmin(admin.ModelAdmin):
    list_display=["name"]
##
## Inline Admin for Attachments
##
class KBEntryAttachmentForm(forms.ModelForm):
    class Meta:
        model=KBEntryAttachment

class KBEntryAttachmentAdmin(admin.TabularInline):
    form=KBEntryAttachmentForm
    model=KBEntryAttachment
    extra=3
##
## Admin for Entries
##
class KBEntryAdmin(admin.ModelAdmin):
    list_display=["id","subject","view_link"]
    search_fields=["id","subject"]
    inlines=[KBEntryAttachmentAdmin]
    def save_model(self, request, obj, form, change):
        obj.save(user=request.user)
##
## Admin for Global Bookmarks
##
class KBGlobalBookmarkAdmin(admin.ModelAdmin):
    list_display=["kb_entry"]
##
## Admin for User Bookmarks
##
class UserBookmarksAdmin(admin.ModelAdmin):
    def queryset(self,request):
        return KBUserBookmark.objects.filter(user=request.user)

    def has_change_permission(self,request,obj=None):
        if obj:
            return obj.has_access(request.user)
        else:
            return admin.ModelAdmin.has_delete_permission(self,request)

    def has_delete_permission(self,request,obj=None):
        return self.has_change_permission(request,obj)
    
##
## Register administrative interfaces
##
admin.site.register(KBCategory, KBCategoryAdmin)
admin.site.register(KBEntry,    KBEntryAdmin)
admin.site.register(KBGlobalBookmark, KBGlobalBookmarkAdmin)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.ip import bits_to_netmask
from noc.lib.registry import Registry
import re
##
##
##
class ProfileRegistry(Registry):
    name="ProfileRegistry"
    subdir="profiles"
    classname="Profile"
    apps=["noc.sa"]
profile_registry=ProfileRegistry()

##
##
##
class ProfileBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        m.scripts={}
        profile_registry.register(m.name,m)
        return m

##
## Abstract Profile
##
class Profile(object):
    __metaclass__=ProfileBase
    # Profile name
    name=None
    #
    # Device capabilities
    #
    
    #
    # A list of supported access schemes.
    # Access schemes constants are defined
    # in noc.sa.protocols.sae_pb2
    # (TELNET, SSH, HTTP, etc)
    #
    supported_schemes=[]
    # Regular expression to catch user name prompt
    # (Usually during telnet sessions)
    pattern_username="([Uu]sername|[Ll]ogin):"
    # Regulal expression to catch password prompt
    # (Telnet/SSH sessions)
    pattern_password="[Pp]assword:"
    # Regular expression to catch command prompt
    # (CLI Sessions)
    pattern_prompt=r"^\S*[>#]"
    # Regular expression to catch unpriveleged mode command prompt
    # (CLI Session)
    pattern_unpriveleged_prompt=None
    # Regular expression to catch pager
    # (Used in command results)
    pattern_more="^---MORE---"
    # Regular expression to catch first pager occurence.
    # (If differ from pattern_more)
    pattern_more_start=None
    # Regular expression to catch last pager occurence.
    # (If differ from pattern_more)
    pattern_more_end=None
    # Sequence to be send at the end of all CLI commands
    #
    command_submit="\n"
    # Sequence to be send to list forward pager
    #
    command_more="\n"
    # Sequence to be send to list pager at first occurence.
    # (If set to None command_more used)
    command_more_start=None
    # Sequence to be send to quit pager when paged to the end
    # (If set to None command_more used)
    command_more_end=None
    # Sequence to gracefully close session
    #
    command_exit="exit"
    # Sequence to enable priveleged mode
    #
    command_super=None
    # List of chars to be stripped out of input stream
    # before checking any regular expressions
    # (when Action.CLEAN_INPUT==True)
    rogue_chars=["\r"]
    ##
    ## Looking glass hilighting support
    ##
    
    # match.group(1) contains AS list
    pattern_lg_as_path_list=None
    # match.group(1) containts Best Path
    pattern_lg_best_path=None
    # Does the equipment supports bitlength netmasks
    # or netmask should be converted to traditional formats
    requires_netmask_conversion=False
    
    #
    # AS Path hilighting
    #
    def lg_as_path(self,m):
        def whois_formatter(q):
            return "<A HREF='http://www.db.ripe.net/whois?AS%s'>%s</A>"%(q,q)
        as_list=m.group(1).split()
        return " ".join([whois_formatter(x) for x in as_list])
    #
    # Converts ipv4 prefix to the format acceptable by router
    #
    def convert_prefix(self,prefix):
        if "/" in prefix and self.requires_netmask_conversion:
            net,mask=prefix.split("/")
            mask=bits_to_netmask(mask)
            return "%s %s"%(net,mask)
        return prefix
    #
    # Leave 00:11:22:33:44:55 style MAC-address untouched
    #
    def convert_mac_to_colon(self,mac):
        return mac
    #
    # Convert 00:11:22:33:44:55 style MAC-address to 0011.2233.4455
    #
    def convert_mac_to_cisco(self,mac):
        v=mac.replace(":","").lower()
        return "%s.%s.%s"%(v[:4],v[4:8],v[8:])
    #
    # Convert 00:11:22:33:44:55 style MAC-address to 00-11-22-33-44-55
    #
    def convert_mac_to_dashed(self,mac):
        v=mac.replace(":","").lower()
        return "%s-%s-%s-%s-%s-%s"%(v[:2],v[2:4],v[4:6],v[6:8],v[8:10],v[10:])
    #
    # Convert 00:11:22:33:44:55 style MAC-address to local format
    # Can be changed in derived classes
    #
    convert_mac=convert_mac_to_colon
    
    #
    # Configuration generators
    #
    
    # Generate prefix list:
    # name - name of prefix list
    # pl -  is a list of prefixes
    # Strict - should tested prefix be exactly matched
    # or should be more specific as well
    #
    def generate_prefix_list(self,name,pl,strict=True):
        raise Excepton("Not implemented")
    #
    # Volatile strings:
    # A list of strings can be changed over time, which
    # can be sweeped out of config safely or None
    # Strings are regexpes, compuled with re.DOTALL|re.MULTILINE
    #
    config_volatile=None
    #
    # Clean up config
    #
    def cleaned_config(self,cfg):
        if self.config_volatile:
            # Wipe out volatile strings before returning result
            for r in self.config_volatile:
                rx=re.compile(r,re.DOTALL|re.MULTILINE)
                cfg=rx.sub("",cfg)
        return unicode(cfg,"utf8","ignore").encode("utf8") # Prevent serialization errors
    #
    # SNMP support
    #
    
    # OID of "config changed trap".
    oid_trap_config_changed=None
    # Regular expression to parse "config changed" syslog message
    syslog_config_changed=None
    #



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

LEGACY=[
    ("Cisco","Cisco.IOS"),
    ("Juniper","Juniper.JUNOS"),
    ("IOS","Cisco.IOS"),
    ("JUNOS","Juniper.JUNOS"),
]
TYPES=["Cisco.IOS","Juniper.JUNOS"]
class Migration:
    def forwards(self):
        for f,t in LEGACY:
            db.execute("UPDATE peer_peeringpointtype SET name=%s WHERE name=%s",[t,f])
        for t in TYPES:
            if db.execute("SELECT COUNT(*) FROM peer_peeringpointtype WHERE name=%s",[t])[0][0]==0:
                db.execute("INSERT INTO peer_peeringpointtype(name) VALUES(%s)",[t])
    
    def backwards(self):
        for f,t in LEGACY:
            db.execute("UPDATE peer_peeringpointtype SET name=%s WHERE name=%s",[t,f])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report
from noc.cm.models import Object
from django import forms
import datetime

class ReportForm(forms.Form):
    repo=forms.ChoiceField(label="Type",choices=[("config","config"),("dns","dns"),("prefix-list","prefix-list")])
    days=forms.IntegerField(label="In Days",min_value=1)

class Report(noc.main.report.Report):
    name="cm.last_changes"
    title="Latest Changes"
    form_class=ReportForm
    columns=[
            Column("Object",format=lambda o:"<A HREF='/cm/view/%s/%s/'>%s</A>"%(o.repo_name,o.id,o.repo_path)),
            Column("Last Changed")
            ]
    
    def get_queryset(self):
        oc=Object.get_object_class(self.form.cleaned_data["repo"])
        baseline=datetime.datetime.now()-datetime.timedelta(days=self.form.cleaned_data["days"])
        r=[(o,o.last_modified) for o in oc.objects.filter(last_modified__gte=baseline).order_by("-last_modified")]
        return r



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## DLink DES3xxx Configuraion-related rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule,Expression,CLOSE_EVENT,DROP_EVENT
from noc.fm.rules.classes.config import *

##
## DLink.DES3xxx Config Changed SYSLOG
##
class DLink_DES3xxx_Config_Changed_SYSLOG_Rule(ClassificationRule):
    name="DLink.DES3xxx Config Changed SYSLOG"
    event_class=ConfigChanged
    preference=1000
    patterns=[
        (r"^source$", r"^syslog$"),
        (r"^profile$",r"^DLink\.DES3xxx$"),
        (r"^message$",r"INFO: Configuration"),
    ]




from south.db import db
from noc.main.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'Language'
        db.create_table('main_language', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True)),
            ('native_name', models.CharField("Native Name",max_length=32)),
            ('is_active', models.BooleanField("Is Active",default=False))
        ))
        
        db.send_create_signal('main', ['Language'])
    
    def backwards(self):
        db.delete_table('main_language')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Remove DNSZoneProfile.zone_ns_list, create necessary DNSServerObjects and links between zones and profiles
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.dns.models import *

class Migration:
    
    def forwards(self):
        for p_id,zl in db.execute("SELECT id,zone_ns_list FROM dns_dnszoneprofile"):
            for n in [x.strip() for x in zl.split(",")]:
                if db.execute("SELECT COUNT(*) FROM dns_dnsserver WHERE name=%s",[n])[0][0]==0:
                    db.execute("INSERT INTO dns_dnsserver(name) values (%s)",[n])
                db.execute("INSERT INTO dns_dnszoneprofile_ns_servers(dnszoneprofile_id,dnsserver_id) SELECT %s,id FROM dns_dnsserver WHERE name=%s",
                    [p_id,n])
                    
        db.delete_column("dns_dnszoneprofile","zone_ns_list")
    
    def backwards(self):
        db.add_column("dns_dnszoneprofile","zone_ns_list",models.CharField("NS list",max_length=64))
        for p in DNSZoneProfile.objects.all():
            p.zone_ns_list=",".join([n.name for n in p.ns_servers.all()])
            p.save()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Fallback classification rules
## Fallback classes are based on event source type only (SNMP Trap, Syslog)
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.fallback import *
##
## Match any unhandled SNMP Trap
##
class UnhandledSNMPRule(ClassificationRule):
    name="Unhandled SNMP"
    event_class=UnhandledSNMPTrap
    preference=100000
    patterns=[
        (r"^source$",                          r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$", r"(?P<oid>\S+)"), # snmpTrapOID.0
    ]
##
## Match any unhandled Syslog message
##
class UnhandledSyslogRule(ClassificationRule):
    name="Unhandled Syslog"
    event_class=UnhandledSyslog
    preference=100000
    patterns=[
        (r"^source$",   r"^syslog$"),
    ]
